#log <- file(snakemake@log[[1]], open="wt")
#sink(log)
#sink(log, type="message")
library(edgeR)
library(tximport)
library(rhdf5)
library(GenomicFeatures)
library(ggplot2)

samples = read.table(snakemake@config[["group_file"]],stringsAsFactors=F)$V1
conditions = read.table(snakemake@config[["group_file"]],stringsAsFactors=F)$V2
files = snakemake@input[["counts"]]
names(files) = samples

if (snakemake@config[["edger_tx2gene"]] == TRUE){
  TxDb <- makeTxDbFromGFF(file = snakemake@config[["edger_annotations"]])
  k <- keys(TxDb, keytype = "TXNAME")
  tx2gene <- select(TxDb, k, "GENEID", "TXNAME")
  txi <- tximport(files, type = snakemake@config[["quantification"]], tx2gene = tx2gene) 
} else {
  txi <- tximport(files, type = snakemake@config[["quantification"]], txOut = TRUE) 
}

cts <- txi$counts
dgelist <- DGEList(cts,group=as.factor(conditions))

#Filtering
countsPerMillion <- cpm(dgelist)
countCheck <- countsPerMillion > 1
keep <- which(rowSums(countCheck) >= 2)
dgelist <- dgelist[keep,]

dgelist <- calcNormFactors(dgelist, method=snakemake@config[["edger_normfact"]])
#plotMDS(dgelist)

groups = as.factor(conditions)
design <- model.matrix(~groups)

if (snakemake@config[["edger_testType"]] == "exactTest"){
        if (snakemake@config[["edger_dispersion"]] == 0){
            dgelist <- edgeR::estimateDisp(dgelist,design=design)
            de.com <- edgeR::exactTest(dgelist)
        }else{
            de.com <- edgeR::exactTest(dgelist, dispersion=snakemake@config[["edger_dispersion"]])
        }
} else if (snakemake@config[["edger_testType"]] == "glmLRT"){
        if (snakemake@config[["edger_dispersion"]] == 0){
            dgelist <- edgeR::estimateDisp(dgelist,design=design)
            fit <- edgeR::glmFit(dgelist)
        } else {
            fit <- edgeR::glmFit(dgelist, dispersion=snakemake@config[["edger_dispersion"]])
        }   
        de.com <- edgeR::glmLRT(fit,coef=2)
}

options(digits=4)

padj<- p.adjust(de.com$table$PValue, method="bonferroni")
res <-data.frame(cbind(de.com$table$logFC/log(2),de.com$table$PValue, padj))
colnames(res) <- c("log2FoldChange", "pvalue", "padj")
rownames(res) <- names(keep)

rm(txi,tx2gene)

save.image(snakemake@output[["r_data"]])
write.csv(res, snakemake@output[["de_table"]])

###### PREPARE REPORT ######

cpm_counts = cpm(cts)
pca = prcomp(t(cpm_counts))

png(filename = paste(snakemake@output[["PCA"]],sep = "/"), height=800, width=800)
ggplot(data.frame(pca$x),aes(x =pca$x[,1] , y = pca$x[,2],col = conditions)) + geom_point()
dev.off()

png(filename = paste(snakemake@output[["MD"]],sep = "/"), height=800, width=800)
plotMD(de.com)
dev.off()
#plotMDS(dgelist, main = "Distances between gene expression profiles")
#plotSmear(lrt, de.tags=tt$table$gene.id)


topGenes = topTags(de.com,n = 40)

cat("# id: Top_genes
# section_name: 'Top 40 genes'
# description: 'Tableau des 40 top genes'
# format: 'tsv'
# plot_type: 'table'\ngene\t", file=snakemake@output[["Top_genes"]])

write.table(as.data.frame(topGenes),snakemake@output[["Top_genes"]],append=TRUE,sep="\t")

png(filename = snakemake@output[["Volcano_plot"]], height=800, width=800)
with(res, plot(log2FoldChange, -log10(pvalue), pch=20, main="Volcano plot", xlim=c(-2.5,2)))
with(subset(res, padj<.05 ), points(log2FoldChange, -log10(pvalue), pch=20, col="red"))
with(subset(res, abs(log2FoldChange)>1), points(log2FoldChange, -log10(pvalue), pch=20, col="orange"))
with(subset(res, padj<.05 & abs(log2FoldChange)>1), points(log2FoldChange, -log10(pvalue), pch=20, col="green"))
dev.off()

cpmTop = cpm_counts[rownames(topGenes$table),]

library(pheatmap)
df <- as.data.frame(conditions)
rownames(df) = samples
colnames(df) = "Conditions"

png(filename = paste(snakemake@output[["Heatmap"]],sep = "/"), height=800, width=800)
pheatmap(cpmTop, cluster_rows=FALSE, show_rownames=FALSE, cluster_cols=FALSE, annotation_col=df)
#heatmap(cpmTop)
dev.off()
#plot_ly(x = colnames(cpmTop),y = rownames(cpmTop),z=cpmTop,type = "heatmap")